@@ -86,21 +86,21 @@ class AttributeDict(dict):
 
         # If there's schema for this object, we will use it to construct object.
         if specification:
-            for key, value in specification['properties'].items():
-                if value.get('type') == 'object':
-                    _data = data.pop(key, {})
-                    self[key] = AttributeDict(data=_data,
-                                              name=key,
-                                              parent=self,
-                                              resource=resource)
-                elif 'relation' in value:
+            for field_name, field_spec in specification['properties'].items():
+                if field_spec.get('type') == 'object':
+                    _data = data.pop(field_name, {})
+                    self[field_name] = AttributeDict(data=_data,
+                                                     name=field_name,
+                                                     parent=self,
+                                                     resource=resource)
+                elif 'relation' in field_spec:
                     pass  # Special handling for relationships
                 else:
-                    self[key] = data.pop(key, None)
+                    self[field_name] = data.pop(field_name, field_spec.get('default'))
+
             if data:
                 logger.warning('There was extra data (not specified in schema): %s',
                                data)
-
         # If not, we will use the source data as it is.
         self.update(data)
         for key, value in data.items():
@@ -275,16 +275,18 @@ class RelationshipDict(dict):
         :param relation_type: either 'to-one' or 'to-many'
         """
         from . import relationships as rel
-        relationship_data = data.get('data')
-        if relationship_data:
+        if 'data' in data:
+            relationship_data = data['data']
             if isinstance(relationship_data, list):
                 if not (not relation_type or relation_type == RelationType.TO_MANY):
                     logger.error('Conflicting information about relationship')
                 return rel.MultiRelationship
-            else:
+            elif relationship_data is None or isinstance(relationship_data, dict):
                 if not(not relation_type or relation_type == RelationType.TO_ONE):
                     logger.error('Conflicting information about relationship')
                 return rel.SingleRelationship
+            else:
+                raise ValidationError('Relationship data key is invalid')
         elif 'links' in data:
             return rel.LinkRelationship
         elif 'meta' in data:
@@ -550,7 +552,7 @@ class ResourceObject(AbstractJsonObject):
 
         url = self._pre_commit(url)
         status, result, location = self.session.http_request(self._http_method, url,
-                                                              self._commit_data(meta))
+                                                             self._commit_data(meta))
         return self._post_commit(status, result, location)
 
     def commit(self, custom_url: str = '', meta: dict = None) \
